(*
 *
 * Copyright (c) 2006-2007, 
 *  Octavian Udrea	<udrea@umiacs.umd.edu>
 *  Cristian Lumezanu	<lume@cs.umd.edu>
 *  Jeffrey S. Foster	<jfoster@cs.umd.edu>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. The names of the contributors may not be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *)

(* File lexer.mll *)
{
open Parser
open Types
let lineStart = ref 0;;
let line = ref 1;;
}

rule token = parse
    [' ']     { token lexbuf }     (* skip blanks *)
  | [';']     { comment lexbuf }     (* skip comments *)
  | ['\t']     { lineStart := !lineStart - 7; token lexbuf; }     (* skip blanks *)
  | ['\n']        { line := !line+1; lineStart := (Lexing.lexeme_end lexbuf); token lexbuf; }
  | ['0'-'9']+     { INT(int_of_string(Lexing.lexeme lexbuf)) }
  | '+'            { PLUS }
  | '.'            { DOT }
  | '-'            { MINUS }
  | '*'            { TIMES }
  | '/'            { DIV }
  | "<="           { LTE }
  | ">="           { GTE }
  | "<>"           { NEQ }
  | '<'            { LT }
  | '>'            { GT }
  | '?'            { QMARK }
  | '='            { EQ }
  | '('            { LPAREN }
  | ')'            { RPAREN }
  | '['            { SQLPAREN }
  | ']'            { SQRPAREN }
  | "$DEFRULE"      { DEFRULE }
  | "$PATTERN"      { PATTERN }
  | "$P_IN"         { PIN }
  | "$IN"           { IN }
  | "$OUT"          { OUT }
  | "$P_OUT"        { POUT }
  | "$SUCCESS"      { SUCCESS }
  | "$FAILURE"      { FAILURE }
  | "$NO_OUTPUT"    { NO_OUTPUT }
  | "$RETURN"       { RETURN }
  | "->"           { ARROW }
  | "=>"           { RULE_ARROW }
  | "$SET!"         { SET }
  | ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']*     { VAR({name = Lexing.lexeme lexbuf}) }
  | eof            { EOF }

and comment = parse
  ['\n']        { line := !line+1; lineStart := (Lexing.lexeme_end lexbuf); token lexbuf; }
  | _                 { comment lexbuf; }
  | eof               { EOF }
