(*
 *
 * Copyright (c) 2006-2007, 
 *  Octavian Udrea	<udrea@umiacs.umd.edu>
 *  Cristian Lumezanu	<lume@cs.umd.edu>
 *  Jeffrey S. Foster	<jfoster@cs.umd.edu>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. The names of the contributors may not be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *)

/* File parser.mly */
%{
open Types;;
%}

%token <int> INT
%token <Types.var> VAR
%token PLUS MINUS TIMES DIV LT GT EQ GTE LTE NEQ
%token LPAREN RPAREN QMARK SQLPAREN SQRPAREN
%token SUCCESS FAILURE NO_OUTPUT DEFRULE ARROW RULE_ARROW
%token SET PATTERN PIN POUT RETURN IN OUT DOT
%token EOL EOF

%start main             /* the entry point */
%type <Types.program> main

%%
main:
PATTERNS RULE_ARROW RULES EOF                { {patterns = $1;
                                    rules = $3} }
  ;

PAT:
    LPAREN PATTERN PIN VAR INT RPAREN          { InPat($4,$5) }
  | LPAREN PATTERN POUT VAR INT RPAREN         { OutPat($4,$5) }
  | LPAREN PATTERN SUCCESS VAR INT RPAREN      { SuccPatF($4,$5) }
  | LPAREN PATTERN SUCCESS RETURN INT RPAREN   { SuccPatR($5) }
  | LPAREN PATTERN FAILURE VAR INT RPAREN         { FailPatF($4,$5) }
  | LPAREN PATTERN FAILURE RETURN INT RPAREN      { FailPatR($5) }
  ;

PATTERNS:
    PAT                       { [$1] }
  | PAT PATTERNS              { $1::$2 }
    ;

IN_VAR:
    IN DOT VAR  { InVar($3) }
  | IN DOT VAR SQLPAREN INT SQRPAREN { ArrayInVar($3,$5) }
    ;

OUT_VAR:
    OUT DOT VAR  { OutVar($3) }
  | OUT DOT VAR SQLPAREN INT SQRPAREN { ArrayOutVar($3,$5) }
    ;

SIMPLE_RULEVAR:
    QMARK VAR { RuleVar($2) }
    ;

RULE_VAR:
    SIMPLE_RULEVAR { $1 }
  | QMARK VAR SQLPAREN INT SQRPAREN { ArrayRuleVar($2,$4) }
    ;

PREMISE_EXPR:
    IN_VAR      { EInVar ($1) }
  | INT 	{ EInt($1) }
  | RULE_VAR    { ERuleVar ($1) }
  | LPAREN PLUS PREMISE_EXPR PREMISE_EXPR RPAREN  { InExpr(PlusOp,$3,$4) }
  | LPAREN MINUS PREMISE_EXPR PREMISE_EXPR RPAREN  { InExpr(MinusOp,$3,$4) }
  | LPAREN DIV PREMISE_EXPR PREMISE_EXPR RPAREN  { InExpr(DivOp,$3,$4) }
  | LPAREN TIMES PREMISE_EXPR PREMISE_EXPR RPAREN  { InExpr(TimesOp,$3,$4) }
  ;

PREMISE:
    LPAREN EQ IN_VAR PREMISE_EXPR RPAREN    { Premise(EqOp,$3,$4) }
  | LPAREN LT IN_VAR PREMISE_EXPR RPAREN    { Premise(LtOp,$3,$4) }
  | LPAREN GT IN_VAR PREMISE_EXPR RPAREN    { Premise(GtOp,$3,$4) }
  | LPAREN NEQ IN_VAR PREMISE_EXPR RPAREN    { Premise(NeqOp,$3,$4) }
  | LPAREN LTE IN_VAR PREMISE_EXPR RPAREN    { Premise(LteOp,$3,$4) }
  | LPAREN GTE IN_VAR PREMISE_EXPR RPAREN    { Premise(GteOp,$3,$4) }
  | LPAREN SIMPLE_RULEVAR VAR RPAREN         { RVarPremiseStr($2,$3) }
  | LPAREN SIMPLE_RULEVAR INT RPAREN         { RVarPremiseInt($2,$3) }
    ;

MIXED_EXPR:
    IN_VAR      { MEInVar($1) }
  | VAR		{ MEVar($1) }
  | INT		{ MEInt($1) }
  | RULE_VAR    { MERuleVar ($1) }
  | OUT_VAR     { MEOutVar($1) }
  | LPAREN PLUS MIXED_EXPR MIXED_EXPR RPAREN  { MixedExpr(PlusOp,$3,$4) }
  | LPAREN MINUS MIXED_EXPR MIXED_EXPR RPAREN  { MixedExpr(MinusOp,$3,$4) }
  | LPAREN DIV MIXED_EXPR MIXED_EXPR RPAREN  { MixedExpr(DivOp,$3,$4) }
  | LPAREN TIMES MIXED_EXPR MIXED_EXPR RPAREN  { MixedExpr(TimesOp,$3,$4) }
  ;

CONCLUSION:
    LPAREN EQ OUT_VAR PREMISE_EXPR RPAREN { Conclusion(EqOp,$3,$4) }
  | LPAREN NEQ OUT_VAR PREMISE_EXPR RPAREN { Conclusion(NeqOp,$3,$4) }
  | LPAREN LT OUT_VAR PREMISE_EXPR RPAREN { Conclusion(LtOp,$3,$4) }
  | LPAREN GT OUT_VAR PREMISE_EXPR RPAREN { Conclusion(LteOp,$3,$4) }
  | LPAREN LTE OUT_VAR PREMISE_EXPR RPAREN { Conclusion(GtOp,$3,$4) }
  | LPAREN GTE OUT_VAR PREMISE_EXPR RPAREN { Conclusion(GteOp,$3,$4) }
  | LPAREN SET SIMPLE_RULEVAR MIXED_EXPR RPAREN { Set($3,$4) }
  | LPAREN NO_OUTPUT RPAREN { NoOut }
  | LPAREN SUCCESS RPAREN { Success }
  | LPAREN FAILURE RPAREN { Fail }

PREMISES:
    PREMISE                   { [$1] }
  | PREMISE PREMISES          { $1::$2 }
  ;

CONCLUSIONS:
    CONCLUSION                { [$1] }
  | CONCLUSION CONCLUSIONS    { $1::$2 }
  ;

RULE:
    LPAREN DEFRULE VAR PREMISES RULE_ARROW CONCLUSIONS RPAREN { {rname = $3;
                                                            premises = $4;
                                                            conclusions = $6;
                                                            } }
    ;

RULES:
    RULE                      { [$1] }
  | RULE RULES                { $1::$2 }
  ;
