(*
 *
 * Copyright (c) 2006-2007, 
 *  Octavian Udrea	<udrea@umiacs.umd.edu>
 *  Cristian Lumezanu	<lume@cs.umd.edu>
 *  Jeffrey S. Foster	<jfoster@cs.umd.edu>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. The names of the contributors may not be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *)

open Eunify
open Rulefunc
open Rmcfind
open Rmcutils
open Rmc
open Types
open Parser
open Pretty
open Cil

let rfile= ref ""
let rname = ref ""

let setRuleFile (x:string)= 
    rfile := x
    ;;

let setRuleName (x:string)=
    rname := x
    ;;

let styleHelp1:string = "The name of the rules file"
let styleHelp2:string = "The rule to check"

let doCheck (f: file) : unit=
    let lexbuf = Lexing.from_channel (Pervasives.open_in !rfile) in
	let prog = Parser.main Lexer.token lexbuf in
	    let c = makeModelCheck prog (!rname) f in
		()
    ;;

let feature : featureDescr = 
  { fd_name = "pistachio";
    fd_enabled = ref true;
    fd_description = "rule-based model checking of network protocols";
    fd_extraopt = 
    [
	("--rulefile", Arg.String (fun x -> setRuleFile x), styleHelp1);
	("--rulename", Arg.String (fun x -> setRuleName x), styleHelp2);
    ];
    fd_doit = doCheck;
    fd_post_check = true
  } 