(*
 *
 * Copyright (c) 2006-2007, 
 *  Octavian Udrea	<udrea@umiacs.umd.edu>
 *  Cristian Lumezanu	<lume@cs.umd.edu>
 *  Jeffrey S. Foster	<jfoster@cs.umd.edu>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. The names of the contributors may not be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *)

open Cil

type var = {name: string}

type arithmeticOp = PlusOp | MinusOp | TimesOp | DivOp
type comparisonOp = EqOp | LtOp | GtOp | NeqOp | LteOp | GteOp

type pattern =
     InPat of var * int
   | OutPat of var * int
   | SuccPatF of var * int
   | FailPatF of var * int
   | SuccPatR of int
   | FailPatR of int

type ruleVar =
    RuleVar of var
  | ArrayRuleVar of var *int

type inVar =
    InVar of var
  | ArrayInVar of var * int

type outVar =
    OutVar of var
  | ArrayOutVar of var * int

type inExpr =
    EInVar of inVar
  | EInt of int
  | ERuleVar of ruleVar
  | InExpr of arithmeticOp * inExpr * inExpr

type mixedExpr =
    MEInVar of inVar
  | MEVar of var
  | MEInt of int
  | MEOutVar of outVar
  | MERuleVar of ruleVar
  | MixedExpr of arithmeticOp * mixedExpr * mixedExpr

type premise =
    Premise of comparisonOp * inVar * inExpr
  | RVarPremiseStr of ruleVar * var
  | RVarPremiseInt of ruleVar * int

type conclusion =
    Conclusion of comparisonOp * outVar * inExpr
  | Set of ruleVar * mixedExpr
  | NoOut
  | Success
  | Fail

type rule =
    { rname: var;
      premises: premise list;
      conclusions: conclusion list
    }

type allVar =
    AInVar of inVar
  | AOutVar of outVar
  | ARuleVar of ruleVar
  | AInt of int
  | AVar of var

type fact =
    Fact of comparisonOp * allVar * mixedExpr
    | Nofact

type factList = fact list

type program =
    { patterns: pattern list;
      rules: rule list
    }

type ruleCFGNode =
    { ruleName: string;
      pred: string list
    }

type ruleCFG = ruleCFGNode list

type stmtFact = int * factList

type ruleEndp = string * (Cil.stmt list)

type ruleContext = 
    { startpoints: Cil.stmt list;
      relVars: allVar list;
      facts: stmtFact list;
      endpoints: ruleEndp list
    }